!******************************************************************************!
!                                                                              !
!    fill_ctparameters.f90                                                     !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   fill_ctparameters.f90                          |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 07/2017                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2020-07-29 15:11             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module fill_ctparameters
  use class_ctparameters
  use modelintegrals, only: set_modelintegrals
  use ctparameter0, only: set_ctparameter0
  use ctparameter1, only: set_ctparameter1
  use ctparameter2, only: set_ctparameter2
  use ctparameter3, only: set_ctparameter3
  use ctparameter4, only: set_ctparameter4
  use derived_ctparameter, only: set_derived_ctparameter
  contains

  subroutine set_renoscheme_mdl(ctparam,renoscheme)
    use input_mdl, only: warning_mdl
    character(*), intent(in) :: ctparam
    character(*), intent(in) :: renoscheme

    select case(ctparam)
    case('dZg_QCD2')
      select case(renoscheme)
      case('onshell')
        dZg_QCD2_scheme = 0
      case default
        call warning_mdl('Invalid renoscheme for '// trim(ctparam) // '.',     &
        where='set_renoscheme_mdl')
      end select 
    case('dZgs_QCD2')
      select case(renoscheme)
      case('Nf6')
        dZgs_QCD2_scheme = 0
      case('Nf5')
        dZgs_QCD2_scheme = 1
      case('Nf4')
        dZgs_QCD2_scheme = 2
      case('Nf3')
        dZgs_QCD2_scheme = 3
      case default
        call warning_mdl('Invalid renoscheme for '// trim(ctparam) // '.',     &
        where='set_renoscheme_mdl')
      end select 
    case default
      call warning_mdl('No renoscheme available for '// trim(ctparam) // '.',  &
      where='set_renoscheme_mdl')
    end select 
  end subroutine set_renoscheme_mdl

  function get_renoscheme_mdl(ctparam)
    use input_mdl, only: warning_mdl
    character(*), intent(in) :: ctparam
    character(len=100) :: get_renoscheme_mdl

    select case(ctparam)
    case('dZg_QCD2')
      select case(dZg_QCD2_scheme)
      case(0)
        get_renoscheme_mdl = "onshell"
      case default
        call warning_mdl('Invalid renoscheme for '// trim(ctparam) // '.',     &
        where='get_renoscheme_mdl')
        get_renoscheme_mdl = "NONE"
      end select 
    case('dZgs_QCD2')
      select case(dZgs_QCD2_scheme)
      case(0)
        get_renoscheme_mdl = "Nf6"
      case(1)
        get_renoscheme_mdl = "Nf5"
      case(2)
        get_renoscheme_mdl = "Nf4"
      case(3)
        get_renoscheme_mdl = "Nf3"
      case default
        call warning_mdl('Invalid renoscheme for '// trim(ctparam) // '.',     &
        where='get_renoscheme_mdl')
        get_renoscheme_mdl = "NONE"
      end select 
    case default
      call warning_mdl('No renoschemes for '// trim(ctparam) // '.', where     &
      ='get_renoscheme_mdl')
      get_renoscheme_mdl = "NONE"
    end select 
  end function get_renoscheme_mdl

  subroutine print_renoschemes_mdl()
    use input_mdl, only: nx
    character(len=100) :: active_renoscheme
    character(len=14) :: fmt = '(4x,a15,a30)'

    write(nx,*) 'Renormalization scheme choice:'
!   dZgs_QCD2, schemes: Nf6,Nf5,Nf4,Nf3
    active_renoscheme = get_renoscheme_mdl("dZgs_QCD2")
    write(nx,fmt) adjustl('dZgs_QCD2: '), trim(active_renoscheme)
    write(nx,'(1x,75("-"))')
  end subroutine print_renoschemes_mdl

  subroutine print_counterterms_mdl()
    use input_mdl, only: nx
    character(len=26) :: fmtc = '(4x,a17,es23.16,x,es23.16)'
    write(nx,'(1x,75("-"))')
    write(nx,*) 'Counterterm parameters'
    write(nx,'(1x,75("-"))')
    write(nx,*) 'Particle counterterm parameter:'
    write(nx,*) 'Particle: g'
    write(nx,fmtc) 'dZg_QCD2 =', dZg_QCD2
    write(nx,'(1x,75("-"))')
    write(nx,*) 'Coupling counterterm parameter:'
    write(nx,fmtc) 'dZgs_QCD2 =', dZgs_QCD2
  end subroutine print_counterterms_mdl

  subroutine set_light_particles_mdl()
    use class_particles
    use input_mdl, only: nx, print_regularization_scheme
    if (print_regularization_scheme) then
      write(nx,'(1x,75("-"))')
      write(nx,*) 'Light fermions renormalization scheme choice:'
    end if 
    if (print_regularization_scheme) write(nx,'(1x,75("-"))')
  end subroutine set_light_particles_mdl

  subroutine init_CTparameters_mdl(reset_collier)
    logical, intent(in) :: reset_collier
    call set_light_particles_mdl
    call set_modelintegrals(reset_collier)
    call set_ctparameter0
    call set_ctparameter1
    call set_ctparameter2
    call set_ctparameter3
    call set_ctparameter4
    call set_derived_ctparameter
  end subroutine init_CTparameters_mdl
end module fill_ctparameters
